/*****************************************************************************
* | File      	:	OLED_Config.c
* | Author      :   Waveshare team
* | Function    :	Hardware interface
* | Info        :	
*----------------
* |	This version:   V1.0
* | Date        :   2019-07-31
* | Info        :   Basic version
******************************************************************************/
#include "OLED_Config.h"
#include <bcm2835.h>
#include <stdio.h>		//printf()

/********************************************************************************
function:	System Init and exit
note:
	Initialize the communication method
********************************************************************************/
char System_Init(void)
{
    if(!bcm2835_init()) {
        printf("bcm2835 init failed   !!! \r\n");
        return -1;
    } else {
        printf("bcm2835 init success  !!! \r\n");
    }

    printf("USE_IIC\r\n");
    bcm2835_i2c_begin();
    bcm2835_i2c_setSlaveAddress(0x3c);  //i2c address
	
	/*
	BCM2835_I2C_CLOCK_DIVIDER_2500 ：2500 = 10us = 100 kHz 
	BCM2835_I2C_CLOCK_DIVIDER_626  ：622 = 2.504us = 399.3610 kHz
	150 = 60ns = 1.666 MHz (default at reset)
	148 = 59ns = 1.689 MHz
	*/	
	bcm2835_i2c_setClockDivider(BCM2835_I2C_CLOCK_DIVIDER_626);
//  bcm2835_i2c_set_baudrate(100000);    //100k baudrate

    return 0;
}

char System_Exit(void)
{
    bcm2835_i2c_end();
    bcm2835_close();
    return 0;
}

/********************************************************************************
function:	Hardware interface
note:
	I2C_Write_Byte(value, cmd):
		hardware I2C
********************************************************************************/
void I2C_Write_Byte(uint8_t value, uint8_t Cmd)
{
    char buf[2] = {Cmd,value};

    int ref = bcm2835_i2c_write(buf, 2);
    while(ref != 0) {
        ref = bcm2835_i2c_write(buf, 2);
        if(ref == 0)
            break;
    }	
}

/********************************************************************************
function:	Delay function
note:
	Driver_Delay_ms(xms) : Delay x ms
	Driver_Delay_us(xus) : Delay x us
********************************************************************************/
void Driver_Delay_ms(uint32_t xms)
{
    bcm2835_delay(xms);
}

void Driver_Delay_us(uint32_t xus)
{
    int j;
    for(j=xus; j > 0; j--);
}
